@@ -6,20 +6,13 @@
 
 #define FIO_HAVE_POSIXAIO
 #define FIO_HAVE_ODIRECT
+#define FIO_USE_GENERIC_BDEV_SIZE
 
 #define OS_MAP_ANON		MAP_ANON
 
 typedef unsigned long os_cpu_mask_t;
 typedef unsigned int os_random_state_t;
 
-/*
- * FIXME
- */
-static inline int blockdev_size(int fd, unsigned long long *bytes)
-{
-	return EINVAL;
-}
-
 static inline int blockdev_invalidate_cache(int fd)
 {
 	return EINVAL;
@@ -48,4 +41,8 @@ static inline long os_random_long(os_random_state_t *rs)
 	return val;
 }
 
+#ifdef MADV_FREE
+#define FIO_MADV_FREE	MADV_FREE
+#endif
+
 #endif
