@@ -12,6 +12,7 @@
 #define FIO_HAVE_POSIXAIO_FSYNC
 #define FIO_HAVE_CPU_AFFINITY
 #define FIO_HAVE_PSHARED_MUTEX
+#define FIO_USE_GENERIC_BDEV_SIZE
 
 #define OS_MAP_ANON		MAP_ANON
 #define OS_RAND_MAX		2147483648UL
@@ -23,14 +24,6 @@ struct solaris_rand_seed {
 typedef psetid_t os_cpu_mask_t;
 typedef struct solaris_rand_seed os_random_state_t;
 
-/*
- * FIXME
- */
-static inline int blockdev_size(int fd, unsigned long long *bytes)
-{
-	return EINVAL;
-}
-
 static inline int blockdev_invalidate_cache(int fd)
 {
 	return EINVAL;
@@ -70,7 +63,7 @@ static inline int fio_set_odirect(int fd)
  * pset binding hooks for fio
  */
 #define fio_setaffinity(pid, cpumask)		\
-	pset_bind(&(cpumask), P_PID, (pid), NULL)
+	pset_bind((cpumask), P_PID, (pid), NULL)
 #define fio_getaffinity(pid, ptr)	({ 0; })
 
 #define fio_cpu_clear(mask, cpu)	pset_assign(PS_NONE, (cpu), NULL)
@@ -105,4 +98,8 @@ static inline int fio_cpuset_exit(os_cpu_mask_t *mask)
  */
 #define FIO_MAX_CPUS			16384
 
+#ifdef MADV_FREE
+#define FIO_MADV_FREE	MADV_FREE
+#endif
+
 #endif
